c-----
c       Program to get power spectral density from SAC files
c       and to form coherency
c       psd [-asc] -S1 suffix1 -S2 suffix2   sac_files ...
c
c       -D    response file is displacement sensitivity (counts/m)
c       -V    response file is velocity     sensitivity (counts/m/s)
c       -A    response file is acceleration sensitivity (counts/m/s/s)
c       -asc       Data files are SAC ASCII FORMAT
c
c       response_file - corresponding amplitude response file
c-----
       parameter (LER=0, LIN=5, LOT=6)
c-----
c       SAC header files
c-----
       common/sachdr/rhdr,ihdr,chdr
       real*4 rhdr(70)
       integer*4 ihdr(40)
       character*8 chdr(24)
c-----
c       define storage space for trace data
c-----
       parameter (LN=32000)
       real*4 x(LN)
       logical sacbin
c-----
c       command line options
c-----
       integer idva
       integer nsac
       character*80 nmsac(50)
       character*80 suffix1, suffix2
       character*80 file1,file2
       character*80 frespa1,frespa2, frespp1, frespp2
c-----
c       AVERAGE PSD ARRAYS
c-----
       integer NARRAY
c   NARRAY must be a power of 2 
       parameter (NARRAY=8192)
       real*4 psd(NARRAY)
       complex z1(NARRAY)
       complex z2(NARRAY)
       real tamp1(NARRAY), tamp2(NARRAY)
       real tpha1(NARRAY), tpha2(NARRAY)
       real*4 tfreq1(NARRAY), tfreq2(NARRAY)
       integer nfreq1, nfreq2
       complex z11(NARRAY), z12(NARRAY), z22(NARRAY)
       real*4 n1(NARRAY), n2(NARRAY), xx(NARRAY)
       real*4 coher(NARRAY)

       logical doadjust
c-----
c       machine dependent initializations
c-----
c-----
c       parse command line options
c-----
       call gcmdln(idva,nsac,nmsac,sacbin,npt2,
     1              suffix1,suffix2,frespa1,frespa2,frespp1,frespp2,
     2              iper,doadjust)
       if(npt2.gt.NARRAY)npt2=NARRAY
       write(LER,*)'idva:     ', idva
       ls = lgstr(frespa1)
       write(LER,*)'frespa1(AMP  ):   ', frespa1(1:ls)
       ls = lgstr(frespp1)
       write(LER,*)'frespp1(PHASE):   ', frespp1(1:ls)
       ls = lgstr(frespa2)
       write(LER,*)'frespa2(AMP  ):   ', frespa2(1:ls)
       ls = lgstr(frespp2)
       write(LER,*)'frespp2(PHASE):   ', frespp2(1:ls)
       write(LER,*)'nsac:     ',nsac
       do 100 i=1,nsac
       ls = lgstr(nmsac(i))
       write(LER,*)'sac(',i,'):', nmsac(i)(1:ls)
 100       continue
       ls1 = lgstr(suffix1)
       ls2 = lgstr(suffix2)
       write(LER,*)'suffix1: ',suffix1(1:ls1),' ',
     1              'suffix2: ',suffix2(1:ls2)
       if(suffix1.eq.'undefined' .or. suffix2.eq.'undefined')then
              call usage(' ')
       endif
c-----
c       get the complex   response for the 
c-----
       call gettamp(frespa1,frespp1,nfreq1,tfreq1,tamp1,tpha1,NARRAY)
       write(LER,*)'tamp1(1)`a',tfreq1(1),tamp1(1),tpha2(1)
       call gettamp(frespa2,frespp2,nfreq2,tfreq2,tamp2,tpha2,NARRAY)
       write(LER,*)'tamp2(1)`a',tfreq2(1),tamp2(1),tpha2(1)
c-----
c       initialize the PSD AVERAGE ARRAYS
c-----
       do 1100 i=1,NARRAY
              z11(i) = cmplx(0.0,0.0)
              z12(i) = cmplx(0.0,0.0)
              z22(i) = cmplx(0.0,0.0)
              n1(i)  = 0.0
              n2(i)  = 0.0
 1100       continue
c-----
c       PROCESS SINGLE TRACE and stack results
c-----
       nseg = 0
       do 2000 ns=1,nsac
              lns=lgstr(nmsac(ns))
              file1=nmsac(ns)(1:lns)//'.'//suffix1(1:ls1)
              file2=nmsac(ns)(1:lns)//'.'//suffix2(1:ls2)
              lf1 = lgstr(file1)
              lf2 = lgstr(file2)
              write(LER,*)'Processing time window: ',ns,
     1                     ' ',file1(1:lf1),' ',file2(1:lf2)
c-----
c       initialize arrays
c-----

              do 2100 i=1,NARRAY
                     psd(i) = 0.0
                     z1(i) = cmplx(0.0,0.0)
                     z2(i) = cmplx(0.0,0.0)
 2100              continue
              call getdat(file1,x,z1,dt1,np1,nerr,npt2,sacbin,
     1                     nfreq1,tfreq1,tamp1,tpha1,idva,NARRAY)
              if(nerr.eq.-1)go to 2001
              call getdat(file2,x,z2,dt2,np2,nerr,npt2,sacbin,
     1                     nfreq2,tfreq2,tamp2,tpha2,idva,NARRAY)
              if(nerr.eq.-1)go to 2001
              if(dt1.ne.dt2 .or. np1.ne.np2)go to 2001
              n21 = np2/2 + 1
              df = 1.0/(np1*dt1)
              nseg = nseg + 1
c-----
c       fill up the matrices
c-----
c-----
c       get PSD from Fourier amplitude spectra, account
c       for negative frequencies (2.0),  Hanning window (2.666667)
c       and window length (n*dt)
c-----
              facpsd = 2.0 * 2.66666667 /(np1*dt1)
              do 2300 i=2,n21-1
                     z11(i) = z11(i) + z1(i)*conjg(z1(i))*facpsd
                     z22(i) = z22(i) + z2(i)*conjg(z2(i))*facpsd
                     z12(i) = z12(i) + z1(i)*conjg(z2(i))*facpsd
 2300              continue
 2001              continue
 2000       continue
       if(doadjust)then
c-----
c              adjust the spectra, assumes that low 
c              signal noise between 6-9 sec
c-----
              adjnum = 0.0
              adjden = 0.0
              df1 = 1.0/(np1*dt1)
              nlw = 0.111111/df1 + 1
              nup = 0.166666/df1 + 1
              do 2301 i = nlw,nup
                     adjnum = adjnum + (cabs(z22(i)))
                     adjden = adjden + (cabs(z11(i)))
 2301              continue
              afac = sqrt(adjnum/adjden)
c-----
c              afac is ratio of gain 1 / gain 2
c              now modify the z22 and z12
c-----
              do 2302 i=2,n21-1
                     z12(i) = z12(i) /afac
                     z22(i) = z22(i)/ (afac*afac)
 2302              continue
              write(6,*)'iper,dt1,afac:',iper,dt1,afac
       endif
c-----
c       do  smoothing
c-----
       call csmooth(z11,z22,z12,n21,coher)
c-----
c       now estimate noise using equations 3.5 3.6 3.3 from
c       
c       Holcomb, G. L. (1989).
c       A Direct Method for Calculating Instrument Noise Levels
c       in Side-by-Side Seismometer Evaluations, 
c       USGS Open-File Report 89-214
c
c       Note that the instrument response has already been removed
c-----
       do 2500 i=2,n21-1,1
              freq = (i-1)*df
              n1(i) = cabs(z11(i)) - cabs(z12(i))
              n2(i) = cabs(z22(i)) - cabs(z12(i))
              xx(i) = cabs(z12(i))
 2500       continue
c-----
c       open output files
c-----
       if(iper.eq.2)then
       open(1,file='Bcoher.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 1
       open(2,file='Bp11.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 2
       open(3,file='Bp22.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 3
       open(4,file='Bn1.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 4
       open(8,file='Bn2.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 8
       open(9,file='Bxx.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 9
       else if(iper.eq.3)then
       open(1,file='Lcoher.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 1
       open(2,file='Lp11.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 2
       open(3,file='Lp22.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 3
       open(4,file='Ln1.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 4
       open(8,file='Ln2.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 8
       open(9,file='Lxx.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 9
       endif
c-----
c       output everything
c-----
       do 3000 i=n21-1,2,-1
              freq = (i-1)*df
              period = 1./freq
c-----
c       FOR BH channel only output periods < 20 seconds
c-----
              if(iper.eq.2 .and. period.le.20.0)then
                write(1,'(2e20.7)')period,coher(i)
                write(2,'(2e20.7)')period,10.0*alog10(cabs(z11(i))/nseg)
                write(3,'(2e20.7)')period,10.0*alog10(cabs(z22(i))/nseg)
                if(n1(i).gt.0.0)then
                write(4,'(2e20.7)')period,10.0*alog10( abs(n1(i))/nseg)
                endif
                if(n2(i).gt.0.0)then
                write(8,'(2e20.7)')period,10.0*alog10( abs(n2(i))/nseg)
                endif
                if(xx(i).gt.0.0)then
                write(9,'(2e20.7)')period,10.0*alog10( abs(xx(i))/nseg)
                endif
c-----
c       FOR LH channel only output periods > 5 sec - we thus get an
c       overlap. Note the BH only had 8192 samples at 20 sps = 400 sec
c       so that the 20 sec period should be stable
c-----
              else if(iper.eq.3 .and. period.ge. 5.0)then
                write(1,'(2e20.7)')period,coher(i)
                write(2,'(2e20.7)')period,10.0*alog10(cabs(z11(i))/nseg)
                write(3,'(2e20.7)')period,10.0*alog10(cabs(z22(i))/nseg)
                if(n1(i).gt.0.0)then
                write(4,'(2e20.7)')period,10.0*alog10( abs(n1(i))/nseg)
                endif
                if(n2(i).gt.0.0)then
                write(8,'(2e20.7)')period,10.0*alog10( abs(n2(i))/nseg)
                endif
                if(xx(i).gt.0.0)then
                write(9,'(2e20.7)')period,10.0*alog10( abs(xx(i))/nseg)
                endif
              endif
 3000       continue
       close(1)
       close(2)
       close(3)
       close(4)
       close(8)
       close(9)
       end

       subroutine smooth(x,n21,np)
       real*4 x(n21)
       integer n21
       nx = np + 1
       mx = (nx -1 ) * 0.75
       mxx = (nx-1)*0.9
       do 1000 m=1,5
              x(m) = (x(m) + x(m+1) + x(m+2) + x(m+3) + x(m+4))/5.0
 1000       continue
       do 2000 m=6,mx
              x(m) = (x(m-1) + x(m-1) + x(m) + x(m+1) + x(m+2))/5.0
 2000       continue
       do 3000 m=mx+1,mxx-1
              x(m) = (x(m-1) + x(m) + x(m+1))/3.0
 3000       continue
       return
       end

       subroutine gcmdln(idva,nsac,nmsac,sacbin,npt2,
     1              suffix1, suffix2,frespa1,frespa2,frespp1,frespp2,
     2              iper,doadjust)
       integer idva
       integer nsac
       character*(*) nmsac(nsac)
       character suffix1*(*), suffix2*(*)
       character frespa1*(*), frespa2*(*)
       character frespp1*(*), frespp2*(*)
       logical sacbin
       logical doadjust
c-----
c       idva       I*4       0 displacment response given
c                     1 velocity response given
c                     2 acceleration reponse given
c       nsac       I*4       Number of SAC files
c       nmsac       Ch()*(*) Array sac file names
c       sacbin       L       .true. SAC binary
c                     .false. SAC ascii
c       suffix1       Ch*(*)
c       suffix2       Ch*(*)
c       frespa1       Ch*(*)       Amplitude 1
c       frespa2       Ch*(*)       Amplitude 2
c       frespp1       Ch*(*)       Phase 1
c       frespp2       Ch*(*)       Phase 2
c       iper       I*4       2 SP ( 20 sps)
c                     3 LP (1sps)
c       doadjust L  .true. ajust spectrum using the 56-9 second range
c-----

       character name*80

       nmarg = mnmarg()
       idva = -1
       nsac = 0
       npt2 = 8192
       i = 0
       iper = 2
       sacbin = .true.
       frespa1 = ' '
       frespp1 = ' '
       frespa2 = ' '
       frespp2 = ' '
       suffix1 = 'undefined'
       suffix2 = 'undefined'
       doadjust = .false.
   10       continue
              i = i + 1
              if(i.gt. nmarg)go to 11
              call mgtarg(i,name)
              if(name(1:2).eq.'-D')then
                     idva = 0
              else if(name(1:2).eq.'-V')then
                     idva = 1
              else if(name(1:2).eq.'-A')then
                     idva = 2
              else if(name(1:3).eq.'-R1')then
                     i = i + 1
                     call getarg(i,frespa1)
              else if(name(1:3).eq.'-R2')then
                     i = i + 1
                     call getarg(i,frespa2)
              else if(name(1:3).eq.'-P1')then
                     i = i + 1
                     call getarg(i,frespp1)
              else if(name(1:3).eq.'-P2')then
                     i = i + 1
                     call getarg(i,frespp2)
              else if(name(1:2).eq.'-N')then
                     i = i + 1
                     call getarg(i,name)
                     read(name,'(bn,i10)')npt2
              else if(name(1:4).eq.'-asc')then
                     sacbin = .false.
              else if(name(1:2).eq.'-?')then
                     call usage(' ')
              else if(name(1:3).eq.'-S1')then
                     i = i + 1
                     call getarg(i,suffix1)
              else if(name(1:3).eq.'-S2')then
                     i = i + 1
                     call getarg(i,suffix2)
              else if(name(1:3).eq.'-SP')then
                     iper = 2
              else if(name(1:3).eq.'-LP')then
                     iper = 3
              else if(name(1:3).eq.'-do')then
                     doadjust = .true.
              else
                     nsac = nsac + 1
                     nmsac(nsac) = name
              endif
       go to 10
   11       continue
       return
       end

       subroutine usage(str)
       character str*(*)

       parameter (LER=0)
       write(LER,*)'Usage: coher [-?] [-D -V -A] -S1 suffix1 '
            write(LER,*)' -S2 suffix2      prefixes ... '
       stop
       end

       function lgstr(str)
c-----
c       function to find the length of a string
c       this will only be used with file system path names
c       thus the first blank 
c       indicates the end of the string
c-----
       character*(*) str
       integer*4 lgstr
       n = len(str)
       lgstr = 1
       do 1000 i=n,1,-1
              lgstr = i
              if(str(i:i).ne.' ')go to 100
 1000       continue
  100   continue
        return
       end


       subroutine gettamp(frespa,frespp,nfreq,tfreq,
     1              tamp,tpha,NRESP)
c-----
c       frespa       Ch*80       name of amplitude response file
c       frespp       Ch*80       name of phase     response file
c       nfreq       I       number of frequency values
c       tfreq       R*4       array of frequencies in increasing order
c       tamp       R*4       array of real amplitude response created
c       tpha       R*4       array of real phase     response created
c       NRESP       I       dimension of the arrays
c-----
       character frespa*(*)
       character frespp*(*)
       real tamp(NRESP), tpha(NRESP)
       real*4 tfreq(NRESP)
       logical ext
       parameter (LER=0, LIN=5, LOT=6)

c-----
c       see of the amplitude response file exists
c-----
       inquire(file=frespa,exist=ext)
        if (.not. ext) then
              call usage('Amplitude response file does not exist')
       endif
c-----
c       see of the phase response file exists
c-----
       inquire(file=frespp,exist=ext)
        if (.not. ext) then
              call usage('Phase response file does not exist')
       endif
c-----
c       open the response file
c       assume that both files are in the order of increasing frequency
c       and have eactly the same number of entries
c-----
       ls = lgstr(frespa)
       open(1,file=frespa(1:ls),form='formatted',
     1              status='unknown',access='sequential')
       rewind 1
       ls = lgstr(frespp)
       open(2,file=frespp(1:ls),form='formatted',
     1              status='unknown',access='sequential')
       rewind 2
       j = 0       
 1000       continue
              read(1,*,end=9990,err=9990)fa,a
              read(2,*,end=9990,err=9990)fp,p
              j = j + 1
              nfreq = j
              tfreq(j) = fa
              tamp(j) = a
              tpha(j) = p
       go to 1000
 9990       continue
       close(1)
       close(2)
       return
       end

       subroutine csmooth(z11,z22,z12,n21,coher)
       complex z11(*), z22(*), z12(*)
       parameter (NARRAY=8192)
       complex tz11(NARRAY), tz12(NARRAY), tz22(NARRAY)
       integer n21
       real coher(*)
       complex zvalxx, zvalyy, zvalxy
       lw = 7
       do 100 i=2,n21-1
              call cavg(z11,i,lw,n21,zvalxx)
              call cavg(z22,i,lw,n21,zvalyy)
              call cavg(z12,i,lw,n21,zvalxy)
              coher(i) = (cabs(zvalxy)*cabs(zvalxy))/
     1                     (cabs(zvalxx) * cabs(zvalyy) )
              tz11(i) = zvalxx
              tz12(i) = zvalxy
              tz22(i) = zvalyy
  100       continue
       do 200 i=1,n21
              z11(i) = tz11(i)
              z12(i) = tz12(i)
              z22(i) = tz22(i)
  200       continue
       return
       end

       subroutine cavg(x,ic,lw,n21,zval)
       integer NPTSER
       parameter (NPTSER=4096)
       complex x(NPTSER), zval
       j = (lw-1)/2
       ilw = ic- j
       iup = ic+ j
       zval = cmplx(0.0,0.0)
c-----
c       safety checks
c-----
       if(ilw.lt.2)then
              ilw=2
              iup = 3 + 2*j
       else if(iup.gt.n21-1)then
              iup = n21-1
              ilw = n21 - 2*j
       endif
       nl = 0
       do 100 i=ilw,iup
              zval = zval + x(i)
              nl = nl + 1
  100       continue
       zval = zval / cmplx(real(nl),0.0)
       return
       end

       subroutine getdat(file,x,z,dt,n,nerr,npt2,sacbin,
     1                     nfreq,tfreq,tamp,tpha,idva,NRESP)
c-----
c       read the data stream from the file,
c       demean, window, compute FFT
c-----
       character file*(*)
       parameter (LN=32000)
       logical sacbin
       real*4 x(LN)
       integer nfreq
       real tfreq(NRESP)
       real tamp(NRESP), tpha(NRESP)
       integer idva
       complex camp
       complex s

       parameter (NARRAY=8192)
       complex z(NARRAY)
              if(sacbin)then
                     call brsac (1,LN,file,x,nerr)
              else
                     call arsac (1,LN,file,x,nerr)
              endif
              if(nerr.ne.-1)then
                     call getnhv('NPTS    ',n,nerr)
                     if(n.lt.npt2)nerr = -1
                     n = npt2
                     call getfhv('DELTA   ',dt,nerr)
                     call dmean(x,n)
                     call hann(x,n)
                     do 2200 i=1,n
                            z(i) = cmplx(x(i),0.0)
 2200                     continue
                     call four(z,n,-1,dt,df)
                     n21 = n/2 + 1
c-----
c       get PSD from Fourier amplitude spectra, account
c       for negative frequencies (2.0),  Hanning window (2.666667)
c       and window length (n*dt)
c-----
                     facpsd = 2.0 * 2.66666667 /(n*dt)
                     do 2300 i=2,n21-1
                            freq = (i-1)*df
                            s = cmplx(0.0, 6.2831853*freq)
                            call resp(freq,camp,nfreq,tfreq,tamp,
     1                                   tpha,NRESP)
c-----
c       convert sensitivity to acceleration sensitivity
c
c       idva       0       instrument response displacement sensitivity
c       idva       1       instrument response velocity     sensitivity
c       idva       2       instrument response acceleration sensitivity
c-----
                            if(idva.eq.0)then
                                   camp = camp/(s*s)
                            else if(idva.eq.1)then
                                   camp = camp/(s)
                            endif
                     z(i) = z(i)/camp
 2300                     continue

              endif
       return
       end

       subroutine resp(freq,camp,nfreq,tfreq,tamp,tpha,NRESP)
c-----
c       get complex response for a given frequency by
c       interpolating the tables
c-----
c       freq       R*4       desired frequency
c       camp       C*8       complex response
c       nfreq       I       number of frequency values
c       tfreq       R*4       array of frequencies in increasing order
c       tamp       R*4       array of real amplitude response created
c       tpha       R*4       array of real phase     response created
c       NRESP       I       dimension of the arrays
c-----
       real*4 freq
       complex camp
       real tamp(NRESP), tpha(NRESP)
       real*4 tfreq(NRESP)

       real p 
c-----
c       assume order from least to largest in frequency
c-----
       do 1000 i=1,nfreq-1
              fup = amax1(tfreq(i),tfreq(i+1))
              flw = amin1(tfreq(i),tfreq(i+1))
              if(freq.ge.flw .and. freq.le.fup)then
                     p = ( freq - tfreq(i))/(tfreq(i+1) - tfreq(i))
                     amp = (1.0-p)*tamp(i) + p*tamp(i+1)
                     pha = (1.0-p)*tpha(i) + p*tpha(i+1)
                     call zresp(camp,  amp, pha)
              endif
 1000       continue
c-----
c       do end member test
c-----
       if(tfreq(1).lt.tfreq(nfreq))then
              if(freq.le.tfreq(1))then
                     call zresp(camp,  tamp(1), tpha(1))
              endif
              if(freq.ge.tfreq(nfreq))then
                     call zresp(camp,  tamp(nfreq), tpha(nfreq))
              endif
       else
              if(freq.ge.tfreq(1))then
                     call zresp(camp,  tamp(1), tpha(1))
              endif
              if(freq.le.tfreq(nfreq))then
                     call zresp(camp,  tamp(nfreq), tpha(nfreq))
              endif
       endif
       return
       end

       subroutine zresp(camp, amp, pha)
c-----
c       create a complex amplitude from an amplitude and phase(degrees)
c-----
c       camp       C*8       Complex response
c       amp       R*4       amplitude response
c       pha       R*4       phase response
c-----
       complex camp
       real amp, pha
       phase = (3.1415927/180.0)*pha
       camp = amp * cmplx(cos(phase),sin(phase))
       return
       end
