c-----
c       Program to get power spectral density from sAC files
c       sacpsd [-asc] [-D -V -A ] -R response_file  sac_files ...
c
c       -D       response file is displacement sensitivity (counts/m)
c       -V       response file is velocity     sensitivity (counts/m/s)
c       -A       response file is acceleration sensitivity (counts/m/s/s)
c       -asc       Data files are SAC ASCII FORMAT
c
c       response_file - corresponding amplitude response file
c-----
       parameter (LER=0, LIN=5, LOT=6)
c-----
c       SAC header files
c-----
       common/sachdr/rhdr,ihdr,chdr
       real*4 rhdr(70)
       integer*4 ihdr(40)
       character*8 chdr(24)
c-----
c       define storage space for trace data
c-----
       parameter (LN=32000)
       common/sacx/x
       real*4 x(LN)
       logical sacbin
c-----
c       command line options
c-----
       integer idva
       integer nsac
       character nmresp*80
       character*80 nmsac(50)
c-----
c       AVERAGE PSD ARRAYS
c-----
       parameter (NARRAY=8192)
       real*4 psdave(NARRAY)
       real*4 psd(NARRAY)
       complex z(NARRAY)
       real*4 tamp(2048)
       real*4 tfreq(2048)
c-----
c       machine dependent initializations
c-----
c-----
c       parse command line options
c-----
       call gcmdln(idva,nmresp,nsac,nmsac,sacbin,npt2)
       if(npt2.gt.NARRAY)npt2=NARRAY
       write(LER,*)'idva:     ', idva
       ls = lgstr(nmresp)
       write(LER,*)'nmresp:   ', nmresp(1:ls)
       write(LER,*)'nsac:     ',nsac
       do 100 i=1,nsac
       ls = lgstr(nmsac(i))
       write(LER,*)'sac(',i,'):', nmsac(i)(1:ls)
 100       continue
c-----
c       get the amplitude response
c-----
       call gettamp(nmresp,nfreq,tfreq,tamp)
c-----
c       convert the  response to acceleration response
c-----
       if(idva.lt.2)then
              do 1000 i=2,nfreq
                     omega = 6.2831853*tfreq(i)
                     if(idva.eq.1)then
                        fac = 1./omega
                     else if(idva.eq.0)then
                        fac = 1./(omega*omega)
                     endif
                     tamp(i) = tamp(i) * fac
 1000              continue
       endif
c-----
c       initialize the PSD AVERAGE ARRAYS
c-----
       do 1100 i=1,NARRAY
              psdave(i) = 0.0
 1100       continue
c-----
c       PROCESS SINGLE TRACE
c-----
       nseg = 0
       do 2000 ns=1,nsac
c-----
c       initialize arrays
c-----

              do 2100 i=1,NARRAY
                     psd(i) = 0.0
                     z(i) = cmplx(0.0,0.0)
 2100              continue
              if(sacbin)then
                     call brsac (1,LN,nmsac(ns),x,nerr)
              else
                     call arsac (1,LN,nmsac(ns),x,nerr)
              endif
              if(nerr.ne.-1)then
                     call getnhv('NPTS    ',n,nerr)
                     if(n.lt.npt2)go to 2001
                     n = npt2
                     call getfhv('DELTA   ',dt,nerr)
                     call dmean(x,n)
                     call hann(x,n)
                     do 2200 i=1,n
                        z(i) = cmplx(x(i),0.0)
 2200                     continue
                     call four(z,n,-1,dt,df)
                        n21 = n/2 + 1
c-----
c       get PSD from Fourier amplitude spectra, account
c       for negative frequencies (2.0),  Hanning window (2.666667)
c       and window length (n*dt)
c-----
                        facpsd = 2.0 * 2.66666667 /(n*dt)
                        do 2300 i=2,n21-1
                        freq = (i-1)*df
                        call resp(freq,amp,nfreq,tfreq,tamp)
                        z(i) = z(i)/amp
                        psd(i) = facpsd *
     1                          (real(z(i))**2 + aimag(z(i))**2 )
                        psdave(i) = psdave(i) + psd(i)
 2300                     continue
                     nseg = nseg + 1
              endif
 2001       continue
 2000       continue
c-----
c       do 5 point smoothing
c-----
       call smooth(psdave,n21,n21)
c-----
c       output everything
c-----
       do 3000 i=n21-1,2,-1
              freq = (i-1)*df
              period = 1./freq
              psdave(i) = psdave(i)/nseg
              if(psdave(i).gt.0.0)then
                     psdpow = 10.0*alog10(psdave(i))
                     write(LOT,'(2e20.7)')period,psdpow
              endif
 3000       continue
       end

       subroutine smooth(x,n21,np)
       real*4 x(n21)
       integer n21
       nx = np + 1
       mx = (nx -1 ) * 0.75
       mxx = (nx-1)*0.9
       do 1000 m=1,5
              x(m) = (x(m) + x(m+1) + x(m+2) + x(m+3) + x(m+4))/5.0
 1000       continue
       do 2000 m=6,mx
              x(m) = (x(m-1) + x(m-1) + x(m) + x(m+1) + x(m+2))/5.0
 2000       continue
       do 3000 m=mx+1,mxx-1
              x(m) = (x(m-1) + x(m) + x(m+1))/3.0
 3000       continue
       return
       end

       subroutine gcmdln(idva,nmresp,nsac,nmsac,sacbin,npt2)
       integer idva
       integer nsac
       character nmresp*(*)
       character*(*) nmsac(nsac)
       logical sacbin

       character name*80

       nmarg = mnmarg()
       idva = -1
       nsac = 0
       npt2 = 8192
       i = 0
       sacbin = .true.
   10       continue
              i = i + 1
              if(i.gt. nmarg)go to 11
              call mgtarg(i,name)
              if(name(1:2).eq.'-D')then
                     idva = 0
              else if(name(1:2).eq.'-V')then
                     idva = 1
              else if(name(1:2).eq.'-A')then
                     idva = 2
              else if(name(1:2).eq.'-R')then
                     i = i + 1
                     call getarg(i,nmresp)
              else if(name(1:2).eq.'-N')then
                     i = i + 1
                     call getarg(i,name)
                     read(name,'(bn,i10)')npt2
              else if(name(1:4).eq.'-asc')then
                     sacbin = .false.
              else if(name(1:2).eq.'-?')then
                     call usage()
              else if(name(1:2).eq.'-h')then
                     call usage()
              else
                     nsac = nsac + 1
                     nmsac(nsac) = name
              endif
       go to 10
   11       continue
       if(idva.lt.0)call usage()
       return
       end

       subroutine usage()
       parameter (LER=0)
       write(LER,*)'Usage: sacpsd [-h] [-D -V -A] -R response_file ',
     1   '       sacfiles ... '
       stop
       end

       function lgstr(str)
c-----
c       function to find the length of a string
c       this will only be used with file system path names
c       thus the first blank 
c       indicates the end of the string
c-----
       character*(*) str
       integer*4 lgstr
       n = len(str)
       lgstr = 1
       do 1000 i=n,1,-1
              lgstr = i
              if(str(i:i).ne.' ')go to 100
 1000       continue
  100   continue
        return
       end

       subroutine resp(freq,amp,nfreq,tfreq,tamp)
       real*4 freq
       real*4 amp
       real*4 tamp(2048)
       real*4 tfreq(2048)
c-----
c       assume order from least to largest
c-----
       do 1000 i=1,nfreq-1
              fup = amax1(tfreq(i),tfreq(i+1))
              flw = amin1(tfreq(i),tfreq(i+1))
              if(freq.ge.flw .and. freq.le.fup)then
                     x = ( freq - tfreq(i))/(tfreq(i+1) - tfreq(i))
                     amp = (1.0-x)*tamp(i) + x*tamp(i+1)
                     return
              endif
 1000       continue
       amp = 1.0e+20
       return
       end

       subroutine gettamp(nmresp,nfreq,tfreq,tamp)
       character nmresp*(*)
       real*4 tamp(2048)
       real*4 tfreq(2048)
       open(1,file=nmresp,form='formatted',
     1              status='unknown',access='sequential')
       rewind 1
       j = 0       
 1000       continue
       
              read(1,*,end=9999,err=9999)fa,a
              j = j + 1
              nfreq = j
              tfreq(j) = fa
              tamp(j) = a
       go to 1000
 9999       continue
       close(1)
       return
       end
