c-----
c       Program to get power spectral density from SAC files
c       and to form coherency
c       sacfilt [-asc] -S1 suffix1 -S2 suffix2   sac_files ...
c
c       -D       response file is displacement sensitivity (counts/m)
c       -V       response file is velocity     sensitivity (counts/m/s)
c       -A       response file is acceleration sensitivity (counts/m/s/s)
c       -asc       Data files are SAC ASCII FORMAT
c
c       response_file - corresponding amplitude response file
c-----
       parameter (LER=0, LIN=5, LOT=6)
c-----
c       SAC header files
c-----
       common/sachdr/rhdr,ihdr,chdr
       real*4 rhdr(70)
       integer*4 ihdr(40)
       character*8 chdr(24)
c-----
c       define storage space for trace data
c-----
       parameter (LN=32000)
       real*4 x(LN)
       logical sacbin
c-----
c       command line options
c-----
       integer idva
       integer nsac
       character*80 nmsac(50)
       character*80 suffix1, suffix2
       character*80 file1,file2
       character*80 fresp1,fresp2
c-----
c       AVERAGE PSD ARRAYS
c-----
       parameter (NARRAY=8192)
       real*4 psd(NARRAY)
       complex z1(NARRAY)
       complex z2(NARRAY)
       complex tamp1(2048), tamp2(2048)
       real*4 tfreq1(2048), tfreq2(2048)
       integer nfreq1, nfreq2
       complex z11(NARRAY), z12(NARRAY), z22(NARRAY)
       real*4 n1(NARRAY), n2(NARRAY)
       real*4 coher(NARRAY)
c-----
c       machine dependent initializations
c-----
c-----
c       parse command line options
c-----
       call gcmdln(idva,nsac,nmsac,sacbin,npt2,
     1              suffix1,suffix2,fresp1,fresp2,iper)
       if(npt2.gt.NARRAY)npt2=NARRAY
       write(LER,*)'idva:     ', idva
       ls = lgstr(fresp1)
       write(LER,*)'fresp1:   ', fresp1(1:ls)
       ls = lgstr(fresp2)
       write(LER,*)'fresp2:   ', fresp2(1:ls)
       write(LER,*)'nsac:     ',nsac
       do 100 i=1,nsac
       ls = lgstr(nmsac(i))
       write(LER,*)'sac(',i,'):', nmsac(i)(1:ls)
 100       continue
       ls1 = lgstr(suffix1)
       ls2 = lgstr(suffix2)
       write(LER,*)'suffix1: ',suffix1(1:ls1),' ',
     1              'suffix2: ',suffix2(1:ls2)
       if(suffix1.eq.'undefined' .or. suffix2.eq.'undefined')then
              call usage()
       endif
c-----
c       get the complex   response for the 
c-----
       call gettamp(fresp1,nfreq1,tfreq1,tamp1,idva)
       call gettamp(fresp2,nfreq2,tfreq2,tamp2,idva)
c-----
c       initialize the PSD AVERAGE ARRAYS
c-----
       do 1100 i=1,NARRAY
              z11(i) = cmplx(0.0,0.0)
              z12(i) = cmplx(0.0,0.0)
              z22(i) = cmplx(0.0,0.0)
              n1(i)  = 0.0
              n2(i)  = 0.0
 1100       continue
c-----
c       PROCESS SINGLE TRACE and stack results
c-----
       nseg = 0
       do 2000 ns=1,nsac
              lns=lgstr(nmsac(ns))
              file1=nmsac(ns)(1:lns)//'.'//suffix1(1:ls1)
              file2=nmsac(ns)(1:lns)//'.'//suffix2(1:ls2)
              lf1 = lgstr(file1)
              lf2 = lgstr(file2)
              write(LER,*)'Processing time window: ',ns,
     1                     ' ',file1(1:lf1),' ',file2(1:lf2)
c-----
c       initialize arrays
c-----

              do 2100 i=1,NARRAY
                     psd(i) = 0.0
                     z1(i) = cmplx(0.0,0.0)
                     z2(i) = cmplx(0.0,0.0)
 2100              continue
              call getdat(file1,x,z1,dt1,np1,nerr,npt2,sacbin,
     1                     nfreq1,tfreq1,tamp1)
              if(nerr.eq.-1)go to 2001
              call getdat(file2,x,z2,dt2,np2,nerr,npt2,sacbin,
     1                     nfreq1,tfreq1,tamp1)
              if(nerr.eq.-1)go to 2001
              if(dt1.ne.dt2 .or. np1.ne.np2)go to 2001
              n21 = np2/2 + 1
              df = 1.0/(np1*dt1)
              nseg = nseg + 1
c-----
c       fill up the matrices
c-----
c-----
c       get PSD from Fourier amplitude spectra, account
c       for negative frequencies (2.0),  Hanning window (2.666667)
c       and window length (n*dt)
c-----
                     facpsd = 2.0 * 2.66666667 /(np1*dt1)
              do 2300 i=1,n21
                     z11(i) = z11(i) + z1(i)*conjg(z1(i))*facpsd
                     z22(i) = z22(i) + z2(i)*conjg(z2(i))*facpsd
                     z12(i) = z12(i) + z1(i)*conjg(z2(i))*facpsd
 2300              continue
 2001              continue
 2000       continue
c-----
c       do  smoothing
c-----
       call csmooth(z11,z22,z12,n21,coher)
c-----
c       now estimate noise
c-----
       do 2500 i=2,n21,1
              freq = (i-1)*df
              n1(i) = cabs(z11(i)) - cabs(z12(i))
              n2(i) = cabs(z22(i)) - cabs(z12(i))
 2500       continue
c-----
c       open output files
c-----
       if(iper.eq.2)then
       open(1,file='Bcoher.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 1
       open(2,file='Bp11.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 2
       open(3,file='Bp22.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 3
       open(4,file='Bn1.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 4
       open(8,file='Bn2.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 8
       else if(iper.eq.3)then
       open(1,file='Lcoher.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 1
       open(2,file='Lp11.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 2
       open(3,file='Lp22.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 3
       open(4,file='Ln1.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 4
       open(8,file='Ln2.out',access='sequential',
     1              status='unknown',form='formatted')
       rewind 8
       endif
c-----
c       output everything
c-----
       do 3000 i=n21-1,2,-1
              freq = (i-1)*df
              period = 1./freq
              if(iper.eq.2 .and. period.le.20.0)then
                     write(1,'(2e20.7)')period,coher(i)
                     write(2,'(2e20.7)')period,10.0*alog10(cabs(z11(i))/nseg)
                     write(3,'(2e20.7)')period,10.0*alog10(cabs(z22(i))/nseg)
                     if(n1(i).gt.0.0)then
                     write(4,'(2e20.7)')period,10.0*alog10( abs(n1(i))/nseg)
                     endif
                     if(n2(i).gt.0.0)then
                     write(8,'(2e20.7)')period,10.0*alog10( abs(n2(i))/nseg)
                     endif
              else if(iper.eq.3 .and. period.ge. 5.0)then
                     write(1,'(2e20.7)')period,coher(i)
                     write(2,'(2e20.7)')period,10.0*alog10(cabs(z11(i))/nseg)
                     write(3,'(2e20.7)')period,10.0*alog10(cabs(z22(i))/nseg)
                     if(n1(i).gt.0.0)then
                     write(4,'(2e20.7)')period,10.0*alog10( abs(n1(i))/nseg)
                     endif
                     if(n2(i).gt.0.0)then
                     write(8,'(2e20.7)')period,10.0*alog10( abs(n2(i))/nseg)
                     endif
              endif
 3000       continue
       close(1)
       close(2)
       close(3)
       close(4)
       close(8)
       end

       subroutine smooth(x,n21,np)
       real*4 x(n21)
       integer n21
       nx = np + 1
       mx = (nx -1 ) * 0.75
       mxx = (nx-1)*0.9
       do 1000 m=1,5
              x(m) = (x(m) + x(m+1) + x(m+2) + x(m+3) + x(m+4))/5.0
 1000       continue
       do 2000 m=6,mx
              x(m) = (x(m-1) + x(m-1) + x(m) + x(m+1) + x(m+2))/5.0
 2000       continue
       do 3000 m=mx+1,mxx-1
              x(m) = (x(m-1) + x(m) + x(m+1))/3.0
 3000       continue
       return
       end

       subroutine gcmdln(idva,nsac,nmsac,sacbin,npt2,
     1              suffix1, suffix2,fresp1,fresp2,iper)
       integer idva
       integer nsac
       character*(*) nmsac(nsac)
       character suffix1*(*), suffix2*(*)
       character fresp1*(*), fresp2*(*)
       logical sacbin

       character name*80

       nmarg = mnmarg()
       idva = -1
       nsac = 0
       npt2 = 8192
       i = 0
       iper = 2
       sacbin = .true.
       suffix1 = 'undefined'
       suffix2 = 'undefined'
   10       continue
              i = i + 1
              if(i.gt. nmarg)go to 11
              call mgtarg(i,name)
              if(name(1:2).eq.'-D')then
                     idva = 0
              else if(name(1:2).eq.'-V')then
                     idva = 1
              else if(name(1:2).eq.'-A')then
                     idva = 2
              else if(name(1:3).eq.'-R1')then
                     i = i + 1
                     call getarg(i,fresp1)
              else if(name(1:3).eq.'-R2')then
                     i = i + 1
                     call getarg(i,fresp2)
              else if(name(1:2).eq.'-N')then
                     i = i + 1
                     call getarg(i,name)
                     read(name,'(bn,i10)')npt2
              else if(name(1:4).eq.'-asc')then
                     sacbin = .false.
              else if(name(1:2).eq.'-?')then
                     call usage()
              else if(name(1:3).eq.'-S1')then
                     i = i + 1
                     call getarg(i,suffix1)
              else if(name(1:3).eq.'-S2')then
                     i = i + 1
                     call getarg(i,suffix2)
              else if(name(1:3).eq.'-SP')then
                     iper = 2
              else if(name(1:3).eq.'-LP')then
                     iper = 3
              else
                     nsac = nsac + 1
                     nmsac(nsac) = name
              endif
       go to 10
   11       continue
       return
       end

       subroutine usage()
       parameter (LER=0)
       write(LER,*)'Usage: coher [-?] [-D -V -A] -S1 suffix1 '
            write(LER,*)' -S2 suffix2      prefixes ... '
       stop
       end

       function lgstr(str)
c-----
c       function to find the length of a string
c       this will only be used with file system path names
c       thus the first blank 
c       indicates the end of the string
c-----
       character*(*) str
       integer*4 lgstr
       n = len(str)
       lgstr = 1
       do 1000 i=n,1,-1
              lgstr = i
              if(str(i:i).ne.' ')go to 100
 1000       continue
  100   continue
        return
       end

       subroutine resp(freq,amp,nfreq,tfreq,tamp)
       real*4 freq
       real*4 amp
       real*4 tamp(2048)
       real*4 tfreq(2048)

       real p 
c-----
c       assume order from least to largest
c-----
       do 1000 i=1,nfreq-1
              fup = amax1(tfreq(i),tfreq(i+1))
              flw = amin1(tfreq(i),tfreq(i+1))
              if(freq.ge.flw .and. freq.le.fup)then
                     p = ( freq - tfreq(i))/(tfreq(i+1) - tfreq(i))
                     amp = (1.0-p)*tamp(i) + p*tamp(i+1)
                     return
              endif
 1000       continue
       amp = 1.0e+20
       return
       end

       subroutine gettamp(fresp,nfreq,tfreq,tamp,idva)
       character fresp*(*)
       real*4 tamp(2048)
       real*4 tfreq(2048)
       ls = lgstr(fresp)
       open(1,file=fresp(1:ls),form='formatted',
     1              status='unknown',access='sequential')
       rewind 1
       j = 0       
 1000       continue
       
              read(1,*,end=9990,err=9990)fa,a
              j = j + 1
              nfreq = j
              tfreq(j) = fa
              tamp(j) = a
              omega = 6.8231853*fa
              if(idva.eq.0)then
                     tamp(j) = tamp(j)/(omega*omega)
              else if(idva.eq.1)then
                     tamp(j) = tamp(j)/(omega)
              endif
       go to 1000
 9990       continue
       close(1)
       return
       end

       subroutine csmooth(z11,z22,z12,n21,coher)
       complex z11(*), z22(*), z12(*)
       integer n21
       real coher(*)
       complex zvalxx, zvalyy, zvalxy
       lw = 7
       do 100 i=1,n21
              call cavg(z11,i,lw,n21,zvalxx)
              call cavg(z22,i,lw,n21,zvalyy)
              call cavg(z12,i,lw,n21,zvalxy)
              coher(i) = (cabs(zvalxy)*cabs(zvalxy))/
     1                     (cabs(zvalxx) * cabs(zvalyy) )
  100       continue
       return
       end

       subroutine cavg(x,ic,lw,n21,zval)
       integer NPTSER
       parameter (NPTSER=4096)
       complex x(NPTSER), zval
       j = (lw-1)/2
       ilw = ic- j
       iup = ic+ j
       zval = cmplx(0.0,0.0)
c-----
c       safety checks
c-----
       if(ilw.lt.1)then
              ilw=1
              iup = 1 + 2*j
       else if(iup.gt.n21)then
              iup = n21
              ilw = n21 - 2*j
       endif
       do 100 i=ilw,iup
              zval = zval + x(i)
  100       continue
       l = 1 + 2*j       
       zval = zval / cmplx(real(l),0.0)
       return
       end

       subroutine getdat(file,x,z,dt,n,nerr,npt2,sacbin,
     1                     nfreq,tfreq,tamp)
c-----
c       read the data stream from the file,
c       demean, window, compute FFT
c-----
       character file*(*)
       parameter (LN=32000)
       logical sacbin
       real*4 x(LN)
       integer nfreq
       real tfreq(*)
       real*4 tamp(*)
       real*4 amp

       parameter (NARRAY=8192)
       complex z(NARRAY)
              if(sacbin)then
                     call brsac (1,LN,file,x,nerr)
              else
                     call arsac (1,LN,file,x,nerr)
              endif
              if(nerr.ne.-1)then
                     call getnhv('NPTS    ',n,nerr)
                     if(n.lt.npt2)nerr = -1
                     n = npt2
                     call getfhv('DELTA   ',dt,nerr)
                     call dmean(x,n)
                     call hann(x,n)
                     do 2200 i=1,n
                            z(i) = cmplx(x(i),0.0)
 2200                     continue
                     call four(z,n,-1,dt,df)
                     n21 = n/2 + 1
c-----
c       get PSD from Fourier amplitude spectra, account
c       for negative frequencies (2.0),  Hanning window (2.666667)
c       and window length (n*dt)
c-----
                     facpsd = 2.0 * 2.66666667 /(n*dt)
                     do 2300 i=2,n21-1
                            freq = (i-1)*df
                            call resp(freq,amp,nfreq,tfreq,tamp)
                            z(i) = z(i)/amp
 2300                     continue

              endif
       return
       end
