c---------------------------------------------------------------------------c
c                                                                           c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                      c
c      VOLUME II                                                            c
c                                                                           c
c      PROGRAM: XYPLT                                                       c
c                                                                           c
c      COPYRIGHT 1988                                                       c
c      D. R. Russell, R. B. Herrmann                                        c
c      Department of Earth and Atmospheric Sciences                         c
c      Saint Louis University                                               c
c      221 North Grand Boulevard                                            c  
c      St. Louis, Missouri 63103                                            c
c      U. S. A.                                                             c
c-----
c       command line flags
c
c       -I      interactive control over plots instead of
c               automatic
c       -W width    line width in inches (0.0)
c       -NOXL       no x-axis label (label is true)
c       -NOYL       no y-axis label (label is true)
c       -NOXN       no x-axis numbers (number is true)
c       -NOYN       no y-axis numbers (number is true)
c       -NDECX ndecx    number of decimal places for x-axis (2)
c       -NDECY ndecy    number of decimal places for y-axis (2)
c-----  
c                                                                           c
c      11/22/88                                                             c
c                                                                           c
c      modifications:                                                       c
c        1)  kdh, 11/88, 5/89, 3/90,                                        c
c            added: figure caption location with symbol;                    c 
c                   plot stand. dev.;                                         c
c                   comments;                                                 c 
c                 legend.                                             c 
c              fixed: errors with drawing axis length,                        c
c                   program abort when input 0 files or wrong file name.    c
c        2)  kdh, 5/91                                                      c
c            added: plot error bars in x-direction (alread had y-direction) c
c        3)  kdh, 11/92                                                     c
c            added: plot boundry limits, fixed SUN compiler bug for         c
c                   implicit do-loop in write statement to binary file.     c
c        4)  kdh, 10/93                                                     c
c            added: plot error bars without symbol (+3 option).             c
c            fixed: legend symbol and text (not quite right).               c
c                   (NOTE: legend information will be a different size from c
c                          the other symbols, axis titles, and numbers,     c
c                          legend scaled by axis length.  MAY want to       c
c                          these other items in the future???.              c
c       rbh 12 17 97 - clipping done properly
c       rbh 02 09 00 - put dashes in legend                                 c
c---------------------------------------------------------------------------c
c
c       Program to plot a data stream of x,y values
c----
        program xyplt3
c
        integer LIN, LOT, LER
      integer NPTSER, NFILE
        parameter (LIN=5, LOT=6, LER=0, ltmp=3)
ccc     parameter (NPTSER=1024, NFILE=50)
        parameter (NPTSER=15000, NFILE=50)
c
        real*4 xp(NPTSER),yp(NPTSER)
        real*4 xe(NPTSER),ye(NPTSER),xe2(NPTSER),ye2(NPTSER)
        real*4 zmin,zplus,cplght
        real*4 linlen(NFILE)
        integer*4 npt(NFILE),isym(NFILE),isty(NFILE),npen(NFILE),
     1  nseg(NFILE),lintyp(NFILE)
        character*80 lablx,lably,name
      character*80 cap(8),clgnd(NFILE)
      logical ext
        logical intact
        logical doxn, doxl,doyl, doyn
        logical isxax
        common/pcntrl/ndecx,ndecy
        integer ndecx, ndecy
c----
c       machine dependent initialization
c----
        call mchdep()
c----
c       parse command line arguments
c----
        call gcmdln(intact,width,doxl,doxn,doyl,doyn,ndecx,ndecy,scln)
        write(6,*)intact,width,doxl,doxn,doyl,doyn
c----
c       begin processing ---------------------------------------
c----
c       select type of plot
c----
  10    continue
                write(LER,*) ' select type of plot:'
                write(LER,*) '       (1) x-y linear'
                write(LER,*) '       (2) x-y logarithmic'
                write(LER,*) '       (3) x-linear, y-logarithmic'
                write(LER,*) '       (4) x-logarithmic, y-linear'
                read(LIN,*) jndx
            write(LER,*)'PLOT TYPE: ',jndx
        if(jndx.lt.1.or.jndx.gt.4) go to 10
c----
c       select type of grid
c----
  20    continue
                write(LER,*) ' select tic, grid or frame:'
                write(LER,*) '       (1) outside tics'
                write(LER,*) '       (2) inside tics'
                write(LER,*) '       (3) frame'
                write(LER,*) '       (4) grid'
                read(LIN,*) igrid
            write(LER,*)'GRID TYPE: ',igrid
        if(igrid.lt.1.or.igrid.gt.4) go to 20
c----
c       interactive mode questions
c----
        if(intact)then
c
c---------- scaling and axis size
c
            write(LER,*) ' enter size factor (0.0 to 1.0)'
            read(LIN,*) size
            write(LER,*)'SIZE     : ', size
            if(size.le.0.0.or.size.gt.1.0) size=0.8
            write(LER,*) ' enter length in inches for x and y axes'
            read(LIN,*) xlen,ylen
            write(LER,*)'XLEN YLEN :',xlen,ylen
   30       continue
                write(LER,*) ' select x and y axes scaling:'
                write(LER,*) '       (0) automatic'
                write(LER,*) '       (1) manual'
                read(LIN,*) isc
                write(LER,*)'SCALING :', isc
            if(isc.lt.0.or.isc.gt.1) go to 30
c
c---------- label titles
c
            write(LER,*) ' input title for x-axis (not over 40 char.)'
            read(LIN,'(a)') lablx
            ntitx=lstr(lablx)
            write(LER,*)'LABEL X: ', lablx(1:ntitx)
            write(LER,*) ' input title for y-axis (not over 40 char.)'
            read(LIN,'(a)') lably
            ntity=lstr(lably)
            write(LER,*)'LABEL Y: ', lably(1:ntity)
c
c---------- caption lines and location
c
   55       write(LER,*) ' Enter number of figure caption lines,'
            write(LER,*) ' max. no. = 8, 0 = no caption.'
            read(LIN,*)icp
            write(LER,*)'CAPTION LINES: ',icp
            if(icp.ne.0)then
                write(LER,*) ' Enter source of caption'
                write(LER,*) ' 0 = type caption interactively'
                write(LER,*) ' 1 = read caption from file' 
                read(LIN,*)  icfile
                write(LER,*)'CAPTION SRC: ',  icfile
            endif
            if(icp.lt.0 .or. icfile.lt.0 .or. icfile.gt.1  
     1          .or. icp.gt.8)go to 55
            if(icp.ne.0)then
                if(icfile.eq.0)then
                    do 60 k=1,icp
                        write(LER,*)'Enter caption',k
                        read(LIN,'(a)') cap(k)
        write(LER,*)'CAPTION(',k,'): ',k,cap(k)
   60               continue
                else
                    read(LIN,'(a)') name
        write(LER,*)'NAME : ',name
                    inquire(file=name,exist=ext)
                    if(.not.(ext)) then
                        write(LER,*) name,' does not exist'
                    else
                        open(2,file=name,status='unknown')
                        rewind (2)
                        do 66 k=1,icp
                            read(2,'(a)',end=67,err=67) cap(k)
   66                   continue
   67                   continue
                        close(2)
                    endif
                endif
                write (LER,*) ' Enter location of caption:'
                write (LER,*) ' ---------------------------  '
                write (LER,*) ' | 0          1          2 | 3'
                write (LER,*) ' |                         |  '
                write (LER,*) ' | 4          5          6 | 7'
                write (LER,*) ' |                         |  '
                write (LER,*) ' | 8          9         10 | 11'
                write (LER,*) ' ---------------------------  '
                read (LIN,*) ipos
                if (ipos.lt.0. or .ipos.gt.11) ipos = 9
                write(LER,*)'Position: ',ipos
            endif
c
c---------- legend information; IF draw interactively 
            
            write(LER,*) ' Desire a legend?  yes = 1, no = 0,'
            write(LER,*) '   (legend info will be input with each', 
     1              ' data file)'
            read(LIN,*) ilgnd
            write(LER,*)'Legend:',ilgnd
            if (ilgnd.ge.1) then
                write (LER,*) ' Enter location for start of legend:'
                write (LER,*) ' ---------------------------  '
                write (LER,*) ' | 0          1          2 | 3'
                write (LER,*) ' |                         |  '
                write (LER,*) ' | 4          5          6 | 7'
                write (LER,*) ' |                         |  '
                write (LER,*) ' | 8          9         10 | 11'
                write (LER,*) ' ---------------------------  '
                read (LIN,*) iclgnd
                if (iclgnd.lt.0. or .iclgnd.gt.11) iclgnd = 9
                write(LER,*)'Legend Location:',iclgnd
            endif
c
c---------- ELSE default all values
c
        else
            icp = 0
            ilgnd = 0
            ipos = 9
            isc = 0
c               nocx=0
            size = 1.0
            xlen=6.0
            ylen=6.0
c               nocy=0
            lablx = 'X'
            lably = 'Y'
        endif
c
        nocx = 0
        nocy = 0
        ntitx = lstr(lablx)
        ntity = lstr(lably)
c----
c       user supplied scaling
c----
c
        if(isc.ne.0) then
            write(LER,*) ' Input min and max values for x-axis'
            read(LIN,*) xf,xl
            write(LER,*)'MIN MAX X-AXIS: ',xf,xl
c           write (LER,*) ' Input no. of major and minor tick,
c    +          'marks for x-axis'
c           read (LIN,*) mjticx,mnticx
            if(jndx.eq.1 .or. jndx.eq.3)then
                xdif = xl - xf
            else
                xdif=alog10(xl)-alog10(xf)
                xf = alog10(xf)
                    xl = alog10(xl)
                nocx = int(xdif)
            endif
            xinc=xdif/xlen
            write(LER,*) ' Input min and max values for y-axis'
            read(LIN,*) yf,yl
            write(LER,*)'MIN MAX Y-AXIS: ',yf,yl
c           write (LER,*) ' Input no. of major and minor tick,
c    +          'marks for y-axis'
c           read (LIN,*) mjticy,mnticy
            if(jndx.eq.1 .or. jndx.eq.4)then
                ydif = yl - yf
            else
                ydif=alog10(yl)-alog10(yf)
                yf=alog10(yf)
                    yl = alog10(yl)
                nocy=int(ydif)
            endif
            yinc=ydif/ylen
        endif
c----
c       input number of files and initialize point defaults
c----
        write(LER,*) ' Input number of files to be plotted on axes',
     1      ' (maximum of ',NFILE,')'
        read(LIN,*) nf
        write(LER,*)'NUMBER FILES: ',nf
        if (nf.le.0) goto 230
c
            xmax=-1.0e20
            ymax=-1.0e20
            xmin=1.0e20
            ymin=1.0e20
c
        open(ltmp,form='unformatted',status='scratch',
     1      access='sequential')
        rewind (ltmp)
c----
c       read in each data file and set parameters
c----
        do 160 i=1,nf
c
  80    write(LER,*) ' Input file name',i,' (no quotes)'
        read(LIN,'(a)') name
        write(LER,*)'FILE NAME :',name
c
            if(intact)then
  110           write(LER,*) ' select line type:'
                write(LER,*)
     +                    '     (-2)  plot symbols with error bars ',
     +                  ' (both x- and y-axes)'
                write(LER,*) '     (-1)  plot symbols only'
                write(LER,*) '     ( 0)  plot line only'
                write(LER,*) '     (+1)  plot symbols and line'
                write(LER,*)
     +                    '     (+2)  plot symbols, line and error ',
     +                  'bars (both x- and y-axes)'
                write(LER,*)
     +                    '     (+3)  plot line and error ',
     +                  'bars (both x- and y-axes) ',
     +                  '(no symbols)'
                read(LIN,*) isty(i)
        write(LER,*)'LINE TYPE : ',isty(i)
                if(isty(i).lt.-2.or.isty(i).gt.3) go to 110
c
                write(LER,*)' Select pen'
                read(LIN,*) npen(i)
        write(LER,*)'PEN COLOR : ',npen(i)
c
                if (isty(i).ge.0) then
                    write(LER,*) ' Select line type:'
                    write(LER,*) '     (0)  solid line '
                    write(LER,*) '     (3 or 21)  short dashes'
                    write(LER,*) '     (7 or 31)  long dashes '
                    read(LIN,*) lintyp(i)
                    if (lintyp(i).ne.3.and.lintyp(i).ne.21.and.
     +              lintyp(i).ne.7.and.lintyp(i).ne.31) 
     +              lintyp(i)=0 
        write(LER,*)'LINE TYPE : ',lintyp(i)
                endif
                if (ilgnd.ge.1) then
                    write(LER,*) 'Enter legend for file:'
                    read(LIN,'(a)') clgnd(i)
        write(LER,*)'FILE LEGEND: ',clgnd(i)
                endif
c
            else
                isty(i) = 0
                npen(i) = 1
                lintyp(i) =0
            endif
            if(lintyp(i).eq.3 .or. lintyp(i).eq.21)then
                linlen(i) = 0.025
            else if(lintyp(i).eq.7 .or. lintyp(i).eq.31)then
                linlen(i) = 0.050
            else
                linlen(i) = 0.10
            endif
            if(isty(i).ne.0 .and. isty(i).ne.3) then
                write(LER,*) ' Select plotting symbol (0 through 13)'
                read(LIN,*) isym(i)
        write(LER,*)'SYMBOL : ',isym(i)
            else
                isym(i) = 0
            endif
c----
c       read in time history in groups of NPTSER points
c       determine maximum for scaling, and write out
c       This give speed of using arrays with low overhead in size
c----
            npt(i) = 0
            nseg(i) = 0
        inquire(file=name,exist=ext)
        if(.not.(ext)) then
            write(LER,*) name,' does not exist'
        else
            open(2,file=name,status='unknown')
            rewind (2)
  141       continue
                mpt = 0
                do 140 j=1,NPTSER
                    if (isty(i).eq.-2.or.isty(i).ge.2) then
                        read (2,*,end=150,err=150) 
     1                  xp(j),yp(j),xe(j),ye(j)
                    else
                        read (2,*,end=150,err=150) xp(j),yp(j)
                    end if
                    npt(i) = npt(i) + 1
                    mpt = mpt + 1
  140           continue
  150           if (mpt.gt.0) then
                    nseg(i) = nseg(i) + 1
                    call amxmn(xp,yp,xe,xe2,ye,ye2,mpt,xmax,xmin,
     +                  ymax,ymin,jndx,isty(i))
                    write (ltmp) mpt
c----------------------
c                   11/20/92 
c                   This code gets around a bug in the 
c                   SUN SC0.0/f77 libraries for doing unformated 
c                   sequential i/o, or in the compiler. 
c----------------------
                    do 155 jl=1,mpt,512
                        ju = (jl - 1 ) + 512 
                        if (ju.gt.mpt) ju = mpt
                        if (isty(i).eq.-2.or.isty(i).ge.2) then
                            write (ltmp) (xp(j),yp(j),xe(j),
     +                      xe2(j),ye(j),ye2(j),j=jl,ju)
                        else
                            write (ltmp) (xp(j),yp(j),j=jl,ju)
                        end if
  155               continue
                endif
                if (mpt.eq.NPTSER) goto 141
                close (2,status='keep')
            endif
  160   continue
c----
c       all data have been read, and the plot extremes determined;
c       initialize plotting, scale and define plot coordinates
c----   
      rewind (ltmp)
      call pinit()
      call factor (size)
      xo = 1.5
      yo = 1.0
      call plot(xo,yo,-3)

      if(isc.ne.1) then
                xl = xmax
                yl = ymax
            call subscl(xmax,xmin,ymax,ymin,jndx,nocx,nocy,
     1      xlen,ylen,xf,xinc,yf,yinc)
        endif
c----
c       draw axes.
c----
        call alogx(xlen,ylen,nocx,nocy,lablx,lably,ntitx,
     1  ntity,xf,yf,xinc,yinc,igrid,doxn,doxl,doyn,doyl,scln)
c----
c       put points on the plot; check for plot limits.
c----
        call gwidth(width)
        call gclip('on',0.0,0.0,xlen,ylen)
        do 200 i=1,nf
            ipen = 3
            do 201 jj=1,nseg(i)
                read (ltmp) mpt
c----------------
c               This code gets around a bug in the 
c               SUN SC0.0/f77 libraries for doing unformated 
c               sequential i/o, or in the compiler.
c----------------
                do 203 jl=1,mpt,512
                    ju = (jl - 1 ) + 512 
                    if (ju.gt.mpt) ju = mpt
                    if (isty(i).eq.-2. or .isty(i).ge.2) then
                        read(ltmp) (xp(j),yp(j),
     +                  xe(j),xe2(j),ye(j),ye2(j),j=jl,ju)
                    else
                        read(ltmp) (xp(j),yp(j),j=jl,ju)
                    endif
  203           continue
                    call newpen (npen(i))
                xp1 = (xp(1)-xf)/xinc
                yp1 = (yp(1)-yf)/yinc
                do 202 j=1,mpt
                    xp2 = (xp(j)-xf)/xinc
                    yp2 = (yp(j)-yf)/yinc
c-----
c       check to see if line segment can be plotted
c-----
                    call rclip(0.0,xlen,0.0,ylen,
     1                  xc0,yc0,xc1,yc1,
     2                  xp1,yp1,xp2,yp2,iplt)
c-----
c       for symbols, see if the symbol or error bars can
c       be plotted
c-----
                    if (xp(j).gt.xl .or. xp(j).lt.xf
     1                  .and.  
     2                  yp(j).gt.yl .or. yp(j).lt.yf) then
                            iplot = 1
                    else
                        iplot = 0
                    endif
c
                    xx = (xp(j)-xf)/xinc
                    yy = (yp(j)-yf)/yinc
c
                if(isty(i).ge.0)then
                    if (lintyp(i).eq.0) then
        if(iplt.gt.0)then
            call plot(xc0,yc0,3)
            call plot(xc1,yc1,2)
            call plot(xc1,yc1,3)
        endif
                    else
        if(iplt.gt.0)then
            call plot(xc0,yc0,3)
                            call plotd(xc1,yc1,lintyp(i),
     1                          linlen(i))
            call plot(xc1,yc1,3)
        endif
        endif
                    endif
                    if (isty(i).ne.0 .and. isty(i).ne.3) then
                        if (iplot.eq.0) then
                            call plot(xx,yy,3)
                            call symbol(xx,yy,0.07,
     1                         char(isym(i)),0.0,-1)
                            call plot(xx,yy,3)
                        endif
                    endif
c
                    if (isty(i).eq.-2.or.isty(i).ge.2) then
c
c----------------------------   y-axis error bars
c
                            call plot(xx,yy,3)
                            if (ye(j).gt.yl)  ye(j)  = yl
                            if (ye(j).lt.yf)  ye(j)  = yf
                            if (ye2(j).gt.yl) ye2(j) = yl
                            if (ye2(j).lt.yf) ye2(j) = yf
                        zplus = (ye(j)-yf)/yinc
                        zmin = (ye2(j)-yf)/yinc
                        call plot(xx-0.035,zplus,3)
                        call plot(xx+0.035,zplus,2)
                        call plot (xx,zplus,3)
                        call plot (xx,zmin,2)
                        call plot (xx-0.035,zmin,3)
                        call plot (xx+0.035,zmin,2)
                        call plot (xx,yy,3)
c
c----------------------------   x-axis error bars
c
                            if (xe(j).gt.xl)  xe(j)  = xl
                            if (xe(j).lt.xf)  xe(j)  = xf
                            if (xe2(j).gt.xl) xe2(j) = xl
                            if (xe2(j).lt.xf) xe2(j) = xf
                        zplus = (xe(j)-xf)/xinc
                        zmin = (xe2(j)-xf)/xinc
                        call plot (zmin,yy-0.035,3)
                        call plot (zmin,yy+0.035,2)
                        call plot (zmin,yy,3)
                        call plot (zplus,yy,2)
                        call plot (zplus,yy-0.035,3)
                        call plot (zplus,yy+0.035,2)
                        call plot (xx,yy,3)
                    end if
c
                    xp1 = xp2
                    yp1 = yp2
  202           continue
                call plot(xx,yy,3)
  201       continue
  200   continue
        call gclip('off',0.0,0.0,xlen,ylen)
        call gwidth(0.0)
c----
c       draw figure caption on plot (if requested)
c----
        if(icp.ne.0) then
            cplght = 0.035*xlen/1.5
            cplght = cplght * scln
            srd = 0.2
            rd = srd
c
            xt = xlen+0.2
            if (ipos.eq.0.or.ipos.eq.4.or.ipos.eq.8) xt=0.2
            if (ipos.eq.1.or.ipos.eq.5.or.ipos.eq.9) xt=xlen/2.
            if (ipos.eq.2.or.ipos.eq.6.or.ipos.eq.10)xt=xlen*2./3.
cccc        yt = ylen-0.2
            yt = ylen - 2.*cplght
            if (ipos.ge.4.and.ipos.le.7)
     1          yt=ylen/2.+(icp+1)*srd/2.-0.07*scln
cccc        if (ipos.ge.8.and.ipos.le.11) yt=0.2+icp*srd
            if (ipos.ge.8.and.ipos.le.11) yt=0.2+cplght+icp*srd
c
c---------- draw caption lines.
c
            do 210 k=1,icp
                lcap = lstr(cap(k))
                call lbox(xt,yt-rd,cap(k),cplght,1)
C               call symbol(xt,yt-rd,cplght,cap(k),0.0,lcap)
                rd=rd+srd
  210       continue
        endif   
c----
c       draw legend for each file on plot (if requested)
c----
        if(ilgnd.ge.1) then
            cplght = 0.035*xlen/1.5
            srd = 0.2
            rd = srd
c
            xt = xlen+0.2
            if (iclgnd.eq.0.or.iclgnd.eq.4.or.iclgnd.eq.8) 
     1          xt=0.2
            if (iclgnd.eq.1.or.iclgnd.eq.5.or.iclgnd.eq.9) 
     1          xt=xlen/2.
            if (iclgnd.eq.2.or.iclgnd.eq.6.or.iclgnd.eq.10)
     1          xt=xlen*2./3.
            yt = ylen-0.2
            if (iclgnd.ge.4.and.iclgnd.le.7)
     1          yt=ylen/2.+(nf+1)*srd/2.-0.07
            if (iclgnd.ge.8.and.iclgnd.le.11) yt=0.2+nf*srd
c
c---------- temporary patch (prior symbol height was 0.1; thus yt
c           moved by 0.2.  forcing everything to be in units of cplght
c           yt = yt*cplght/0.2  
c
c---------- draw legend
c
            do 215 k=1,nf
                    call newpen (npen(k))
ccc                 xsym = xt+cplght
                xsym = xt
                ysym = yt-rd+cplght
                xlnon = xt-0.20
ccc                 xlnoff = xt+cplght+0.05
                xlnoff = xt+0.10
                if (isty(k).eq.0) then
                    call gwidth(width)
                    call plot (xlnon,ysym,3)
C 02/09/00              call plot (xlnoff,ysym,2)
                    if(lintyp(k).eq.0)then
                        call plot (xlnoff,ysym,2)
                    else
                        call plotd(xlnoff,ysym,lintyp(k),
     1                          linlen(k))
                    endif
                    call  gwidth(0.0)
                elseif (isty(k).gt.0) then
                    call plot (xlnon,ysym,3)
C 02/09/00              call plot (xlnoff,ysym,2)
                    if(lintyp(k).eq.0)then
                        call plot (xlnoff,ysym,2)
                    else
                        call plotd(xlnoff,ysym,lintyp(k),
     1                          linlen(k))
                    endif
                    if (isty(k).ne.3) then
                        call symbol(xsym,ysym,cplght,char(isym(k)),
     +                  0.0,-1)
                    endif
                elseif (isty(k).lt.0) then
                    call symbol(xsym,ysym,cplght,char(isym(k)),0.0,-1)
                endif
c
                call newpen (npen(1))
                call newpen(1)
ccc             call symbol(xlnoff+cplght,yt-rd,cplght,clgnd(k),0.0,80)
                ls = lgstr(clgnd(k))
                call symbol(xlnoff+cplght,ysym-cplght/2.,0.8*cplght,
     +          clgnd(k)(1:ls),0.0,ls)
                rd=rd+srd
  215       continue
        endif
c
c----   move the cursor out of the way (mainly for CRT plotting)
c
        call plot(xo+xlen,yo+ylen,3)
c
        call newpen(1)
        call pend()
        close (ltmp,status='delete')
  230   stop         
        end
c
c
c
        subroutine amxmn(x,y,xe,xe2,ye,ye2,npt,xmax,xmin,
     +      ymax,ymin,jndx,ist)   
c----
c       modified:  kdh, 11/22/88, 5/91
c----
        integer LIN, LOT, LER
ccc     parameter (LIN=5, LOT=6, LER=0, MAX=1024)
        parameter (LIN=5, LOT=6, LER=0, MAX=15000)
ccc     real*4 x(1),y(1),xe(1),xe2(1),ye(1),ye2(1)
        real*4 x(MAX),y(MAX),xe(MAX),xe2(MAX),ye(MAX),ye2(MAX)
        integer ist
c
c**x-axis; for each data point:
c
c----   check for logarithmic scaling.
c
        if (jndx.eq.2.or.jndx.eq.4) then
            do 10 j=1,npt    
c
c----------------   on error bars (if any).
c
                if (ist.eq.-2.or.ist.ge.2) then
                    if(xe(j).le.0.0) then
                    write(LER,*) 'warning : trying to find',
     1                  'logarithm of zero or negative value.',
     2                  ' xe(',j,') = ',xe(j)
                    xe(j) = 0.0
                    endif
                    xe2(j) = alog10(x(j) - xe(j))
                    xe(j) = alog10(x(j) + xe(j))
                endif
c
c----------------   and on data point.
c
                if (x(j).le.0.0) then
                    write(LER,*) 'warning : trying to find',
     1              'logarithm of zero or negative',
     2              'value.  x(',j,') = ',x(j)
                endif
                x(j) = alog10(x(j))  
   10       continue
c
c----   check for linear scaling on error bars (if any).
c
        else
            if (ist.eq.-2.or.ist.ge.2) then
                do 20 j=1,npt
                    xe2(j) = x(j) - xe(j)
                    xe(j) = x(j) + xe(j)
   20           continue
            endif
        endif
c
c**y-axis; for each point;
c
c----   check for logarithmic scaling.
c
        if (jndx.eq.2.or.jndx.eq.3) then
            do 30 j=1,npt     
c
c----------------   and on error bars (if any).
c
                if (ist.eq.-2.or.ist.ge.2) then
                    if(ye(j).le.0.0) then
                    write(LER,*) 'warning : trying to find',
     1                  'logarithm of zero or negative value.',
     2                  ' ye(',j,') = ',ye(j)
                    ye(j) = 0.0
                    endif
                    ye2(j) = alog10(y(j) - ye(j))
                    ye(j) = alog10(y(j) + ye(j))
                endif
c
c----------------   and on data point.
c
                if (y(j).le.0.0) then
                    write(LER,*) 'warning : trying to find',
     1              'logarithm of zero or negative value.',
     2                  ' y(',j,') = ',y(j)
                endif
                y(j) = alog10(y(j))
   30       continue
c
c----   check for linear scaling on error bars (if any).
c
        else
            if (ist.eq.-2.or.ist.ge.2) then
                do 40 j=1,npt
                    ye2(j) = y(j) - ye(j)
                    ye(j) = y(j) + ye(j)
   40           continue
            endif
        endif
c
c**each point is now scaled; determine maximum and minimum point
c**to scale each axes.
c
        do 50 j=1,npt
            xt=x(j)
            yt=y(j)
            if(xt.gt.xmax) xmax = xt    
            if(yt.gt.ymax) ymax = yt 
            if(xt.lt.xmin) xmin = xt 
            if(yt.lt.ymin) ymin = yt
c
            if (ist.eq.-2.or.ist.ge.2) then
                xp = xe(j)
                xm = xe2(j)
                if(xp.gt.xmax) xmax = xp 
                if(xm.lt.xmin) xmin = xm
c
                yp = ye(j)
                ym = ye2(j)
                if(yp.gt.ymax) ymax = yp 
                if(ym.lt.ymin) ymin = ym
            end if
  50    continue
c
        return       
        end
c
c
c

        subroutine subscl(xmax,xmin,ymax,ymin,jndx,nocx,nocy,
     1                   xlen,ylen,xf,xinc,yf,yinc)
        dimension x(4),y(4)
c
        x(1)=xmin
        x(2)=xmax
        y(1)=ymin
        y(2)=ymax
        xmax=int(xmax)+1
        xmin=int(xmin)-1
        ymax=int(ymax)+1
        ymin=int(ymin)-1
        nocx=int(xmax-xmin)
        nocy=int(ymax-ymin)
c
        if(jndx.eq.1)then
            call gscale(x,xlen,2,1)
            call gscale(y,ylen,2,1)
            xf=x(3)
            xinc=x(4)
            yf=y(3)
            yinc=y(4)
            nocx=0
            nocy=0
        elseif(jndx.eq.2)then
            xf=xmin
            xinc=(xmax-xmin)/xlen
            yf=ymin
            yinc=(ymax-ymin)/ylen
        elseif(jndx.eq.3)then
            yf=ymin
            yinc=(ymax-ymin)/ylen
            call gscale(x,xlen,2,1)
            nocx=0
            xf=x(3)
            xinc=x(4)
        else
            xf=xmin
            xinc=(xmax-xmin)/xlen
            call gscale(y,ylen,2,1)
            yf=y(3)
            yinc=y(4)
            nocy=0
        endif
c
        return
        end
c
c
c
        subroutine alogx(xaxlen,yaxlen,nocx,nocy,ttlx,ttly,    
     1  mtx,mty,x1,y1,deltax,deltay,igrid,doxn,doxl,doyn,doyl,scln)
      character ttlx*(*),ttly*(*)        
        logical doxn, doxl,doyl, doyn
        logical isxax
        common/pcntrl/ndecx,ndecy
        integer ndecx, ndecy
c
      if(igrid.eq.4) then
            rl=1.0
            rs=1.0
      else
            rl=0.02
            rs=0.01
      endif
      if(igrid.ne.1)then
             sgn=+1.0
      else
             sgn=-1.0
      endif
      slt = rl * yaxlen
      sst = rs * yaxlen    
      sp = -0.06*yaxlen      
      ss = 0.035*yaxlen      

ccc         ssp = sp + ss - 0.06   
      ssp = sp + ss -0.03
      ttlp = -0.11*yaxlen - 0.1        
      sttl = 0.035*yaxlen    
c
      if(igrid.eq.1) then
            call plot(-0.02*yaxlen,0.0,2)
            call plot(0.0,-0.02*yaxlen,3)
            call plot(0.0,0.0,2)
      endif
      xpo = x1     
      ypo = y1     
      if(nocx.gt.0) then
            anocx = nocx 
            factx = xaxlen/anocx   
            if(doxn)then
ccc             call symbol(-0.6*ss,sp,ss,'10',0.0,2)       
            call symbol(-0.6*ss,sp-0.025,ss,'10',0.0,2)       
ccc             call number(999.,ssp,0.5*ss,x1,0.0,-1)     
            call number(999.,ssp-0.025,0.7*ss,x1,0.0,-1)     
            endif
            call plot(0.0,0.0,3)   
            do 40 j = 1,nocx        
                  do 30 i = 1,10
                        x = i        
                        x = alog10(x) *factx + (j-1)*factx         
                        if(i.gt.1)then
                              call plot(x,0.0,2)     
                              call plot(x,sgn*sst,2)
                              if(igrid.eq.3) then
                                    call plot(x,yaxlen-sst,3)
                                    call plot(x,yaxlen,2)
                              endif
                        endif
                        call plot(x,0.0,3)     
   30             continue
                  call plot(x,sgn*slt,2)
                  if(igrid.eq.3) then
                        call plot(x,yaxlen-slt,3)
                        call plot(x,yaxlen,2)
                  endif
                  xpo = xpo + 1.0        
            if(doxn)then
ccc                 call symbol(x-0.6*ss,sp,ss,'10',0.0,2)      
                  call symbol(x-0.6*ss,sp-0.03,ss,'10',0.0,2)      
ccc                 call number(999.,ssp,0.5*ss,xpo,0.0,-1)    
                call number(999.,ssp-0.025,0.7*ss,xpo,0.0,-1)     
            endif
                  call plot(x,0.0,3)     
   40       continue
            if(igrid.ge.3)then
                  call plot(0.0,yaxlen,3)
                  call plot(x,yaxlen,2)
                  call plot(x,0.0,3)
            endif
            xtl = mtx    
            xtl = (xaxlen-xtl*sttl)/2.0      
            if(doxl)call symbol(xtl,ttlp,sttl,ttlx,0.0,mtx)    
      else
            isxax = .true.
            call axisg(0.0,0.0,ttlx,-mtx,xaxlen,0.0,x1,deltax,    
     1            yaxlen,igrid,doxn,doxl,doyn,doyl,isxax,ndecx,scln)
      endif
      call plot(0.0,0.0,3)   
c
c-------------------------
c     BUG -- kdh, 09/30/93
c           need to reset the 'slt' and 'sst' variables; not working
c           when x-axis length is not the same as the y-axis length.
c
      slt = rl * xaxlen
      sst = rs * xaxlen    
c
c-------------------------
c
      if(nocy.gt.0) then
            anocy = nocy 
            xnum = 1.0
            yl = y1
            yu = y1 + anocy
            if(abs(yl).ge.10.0.or.abs(yu).ge.10.0) xnum = xnum + 1.0
            if(abs(yl).ge.100..or.abs(yu).ge.100.) xnum = xmun + 1.0
            if(y1.lt.0.0) xnum = xnum + 1.0
            sp = sp - (xnum - 1.5) * 0.5 * ss
            ttlp = ttlp - (xnum-1.0)*0.5*ss   
            facty = yaxlen/anocy   
            if(doyn)then
ccc             call symbol(sp-0.4,-0.5*ss,ss,'10',0.0,2)  
            call symbol(sp-0.3,-0.5*ss,ss,'10',0.0,2)  
ccc             call number(999.,0.5*ss-0.06,0.5*ss,y1,0.0,-1)
            call number(999.,0.5*ss-0.02,0.7*ss,y1,0.0,-1)
            endif
            call plot(0.0,0.0,3)   
            do 80 j = 1,nocy        
                  do 70 i = 1,10
                        y = i        
                        y = alog10(y) * facty + (j-1)*facty        
                        if(i.gt.1)then
                              call plot(0.0,y,2)     
                              call plot(sgn*sst,y,2)
                              if(igrid.eq.3) then
                                    call plot(xaxlen-sst,y,3)
                                    call plot(xaxlen,y,2)
                              endif
                    endif
                        call plot(0.0,y,3)     
   70             continue
                  call plot(sgn*slt,y,2)
                  if(igrid.eq.3) then
                              call plot(xaxlen-slt,y,3)
                              call plot(xaxlen,y,2)
                  endif
ccc                 call symbol(sp-0.4,y-0.5*ss,ss,'10',0.0,2)   
                  ypo = ypo + 1.0
            if(doyn)then
                  call symbol(sp-0.3,y-0.5*ss,ss,'10',0.0,2)   
ccc                 call number(999.,y+0.5*ss-0.06,0.5*ss,ypo,0.0,-1)       
                call number(999.,y+0.5*ss-0.02,0.7*ss,ypo,0.0,-1)
            endif
                  call plot(0.0,y,3)     
   80       continue
            if(igrid.ge.3)then
                  call plot(xaxlen,0.0,3)
                  call plot(xaxlen,y,2)
                  call plot(0.0,y,3)
            endif
            ytl=mty      
            ytl = (yaxlen-ytl*sttl)/2.0      
            if(doyl)call symbol(ttlp-0.2,ytl,sttl,ttly,90.0,mty) 
      else       
            isxax = .false.
            call axisg(0.0,0.0,ttly,mty,yaxlen,90.,y1,deltay, 
     1           xaxlen,igrid,doxn,doxl,doyn,doyl,isxax,ndecy,scln)
      endif
      return       
      end
c
c
c
      subroutine axisg(xpage,ypage,ibcd,nchar,axlen,angle,firstv,
     1     deltav,axisot,igrid,doxn,doxl,doyn,doyl,isxax,ndec,scln)
c.....     xpage,ypage  coordinates of starting point of axis, in inche
c.....     ibcd         axis title.
c.....     nchar        number of characters in title. + for c.c-w side
c.....     axlen        floating point axis length in inches.
c.....     angle        angle of axis from the x-direction, in degrees.
c.....     firstv       scale value at the first tic mark.
c.....     deltav       change in scale between tic marks one inch apart
c.....     igrid        gridding flag, 1- no grid, 3- just frame, 4- grid
c.....     axisot       other axis length for framing
c-----
c       doxn    L   put numbers on X-axis
c       doyn    L   put numbers on Y-axis
c       doxl    L   put labels  on X-axis
c       doyl    L   put labels  on Y-axis
c       isxax   L   .true. this is X axis else Y axis
c-----
        logical doxn, doxl,doyl, doyn
      character ibcd*(*)
        logical isxax
        logical dolab, donum
        logical isneg
        write(6,*)doxn,doyn,doxl,doyl,donum,dolab
        if(isxax)then
            if(doxn)then
                donum = .true.
            else
                donum = .false.
            endif
            if(doxl)then
                dolab = .true.
            else
                dolab = .false.
            endif
        else
            if(doyn)then
                donum = .true.
            else
                donum = .false.
            endif
            if(doyl)then
                dolab = .true.
            else
                dolab = .false.
            endif
        endif
        write(6,*)doxn,doyn,doxl,doyl,donum,dolab
c-----
c       get placement  for number
c-----
        if(nchar.ge.0)then
            a = 1.0
            kn = nchar
        else
            a = -1.0
            kn = -nchar
        endif
c-----
c       get power of 10 for number
c       and adjust the first value
c-----
        ht = 0.105*scln
      ex=0.0
      adx= abs  (deltav)
      if (adx) 3,7,3
  3   if (adx- 99.0) 6,4,4
  4   adx=adx/10.0
      ex=ex+1.0
      go to 3
  5   adx=adx*10.0
      ex=ex-1.0
  6   if (adx-0.01) 5,7,7
  7   xval=firstv*10.0**(-ex)
      adx= deltav*10.0**(-ex)
      sth=angle*0.0174533
      cth=cos(sth)
      sth=sin(sth)
      dxb=-0.1
        dxb = dxb*scln
      dyb=0.15*a-0.05
        dyb = dyb * scln
      xn=xpage+dxb*cth-dyb*sth
      yn=ypage+dyb*cth+dxb*sth
      ntic=axlen+1.0
      nt=ntic/2
      do  20  i=1,ntic
        if(donum)then
c-----
c       for i=1 and i = ntic adjust position
c-----
c       get size of number the above code allocates no more
c       than a number of (+-)XX.XX
c-----
            npos=2
            if(xval.lt.0.0)then
                axval = -xval
                isneg = .true.
                npos = npos + 1
            else
                axval =  xval
                isneg = .false.
            endif
            if(axval.gt.10)then
                npos = npos + ndec
            else 
                npos = npos + ndec -1
            endif
            if(i.eq.1)then
            call number(xn-dxb*cth,yn-dxb*sth,ht,xval,angle,ndec)
            else if(i.eq.ntic)then
            call number(xn-npos*ht*cth,yn-npos*ht*sth,
     1          ht,xval,angle,ndec)
            else
            call number(xn,yn,ht,xval,angle,ndec)
            endif
        endif
        xval=xval+adx
c-----
c       the numbers are placed every inch really
c       xn=xn+cth*1.0
c       yn=yn+sth*1.0
c-----
      xn=xn+cth
      yn=yn+sth
      if (nt) 20,11,20
 11   z=kn
      if (ex)  12,13,12
 12   z=z+7.0
 13   dxb=-.07*z+axlen*0.5
            dxb = dxb * scln
      dyb=0.325*a-0.075
            dyb = dyb *scln
      xt=xpage+dxb*cth-dyb*sth
      yt=ypage+dyb*cth+dxb*sth
        if (angle.ne.0.0) xt=xt-0.1*scln
        if(dolab)then
            call symbol(xt,yt,0.14*scln,ibcd,angle,kn)
        endif
      if (ex .ne.0.0)  then
            z=kn+2-1
            xt=xt+z*cth*0.14
            yt=yt+z*sth*0.14
            xxt=xt+(3.0*cth-0.8*sth)*0.14
            xyt=yt+(3.0*sth+0.8*cth)*0.14
            if(donum)then
            call symbol(xt,yt,0.14,'*10',angle,3)
            call number(xxt,xyt,0.07,ex,angle,-1)
            endif
        endif
 20   nt=nt-1
      gridf = 0.07
      if (igrid .eq. 4) gridf = axisot
      call plot(xpage+axlen*cth,ypage+axlen*sth,3)
      if(igrid.lt.2) go to 22
      inticx = +1
      inticy = -1
      go to 23
 22   continue
      inticx = -1
      inticy = +1
 23   continue
      dxb=inticx*gridf*a*sth
      dyb=inticy*gridf*a*cth
      a=ntic-1
      xn=xpage+a*cth
      yn=ypage+a*sth
      savxn = xn
      savyn = yn
c----
c       draw tic marks
c----
      do  30  i=1,ntic
        call plot(xn,yn,2)
        call plot(xn+dxb,yn+dyb,2)
        if(igrid.ne.3) go to 27
c
c---- draw tics on entire frame
c
        if(angle.eq.90.0) go to 25
        call plot(xn,yn+axisot-dyb,3)
        call plot(xn,yn+axisot,2)
        go to 26
  25    continue
        call plot(xn+axisot-dxb,yn,3)
        call plot(xn+axisot,yn,2)
  26    continue
        call plot(xn,yn,3)
        go to 28
  27    continue
c
c---- retrace line
c
        call plot(xn,yn,2)
  28    continue
        xn=xn-cth
        yn=yn-sth
 30   continue
c
c---- if selected frame, draw lines to match the tic marks
c
      if(igrid.ge.3) then
        if(angle.eq.90.0) then
            call plot(axisot,0.0,3)
c           call plot(axisot,savyn,2)
            call plot(axisot,axlen,2)
        else
            call plot(0.0,axisot,3)
c           call plot(savxn,axisot,2)
            call plot(axlen,axisot,2)
        endif
      endif
c
      return
      end

      subroutine usage(ierr)
      integer LER
      parameter (LER=0)
      integer ierr
c----
c     prompt for proper usage
c----
        if(ierr.eq.0)then
            write(LER,*)'Usage: xyplt -I '
        elseif(ierr.eq.-1)then
            write(LER,*)'xyplt: unexpected EOF on input'
        elseif(ierr.eq.-2)then
            write(LER,*)'xyplt: read error on input'
        endif
      stop
      end
c
c
c
      subroutine gcmdln(intact,width,doxl,doxn,doyl,doyn,ndecx,ndecy,
     1  scln)
c----
c     parse command line arguments and return control
c     parameters
c
c     requires subroutine mgtarg(i,name) to return
c           the i'th argument in the string name
c
c     and the function mnmarg() to return the number
c           of arguments excluding program name
c           The first argument is i = 1
c
c----
c       intact  L   - interactive plot scaling
c       width   R*4 - line width
c       scln    R*4 - scale numbers by this factor
c----
        logical intact
      character*50 name
      integer*4 mnmarg
        logical doxl, doxn, doyl, doyn
c----
c     set up defaults for poor usage test
c----
        intact = .false.
        width = 0.0
        doxl = .true.
        doyl = .true.
        doxn = .true.
        doyn = .true.
        ndecx = 2
        ndecy = 2
        scln = 1.0
      nmarg = mnmarg()
      i = 0
   11 i = i + 1
      if(i.gt.nmarg)goto 13
            call mgtarg(i,name)
            if(name(1:2).eq.'-I')then
                intact = .true.
            else if(name(1:2).eq.'-W')then
                i = i + 1
                    call mgtarg(i,name)
                read(name,'(f10.0)')width
                if(width.lt.0.0)width = 0.0
            else if(name(1:2).eq.'-S')then
                i = i + 1
                    call mgtarg(i,name)
                read(name,'(f10.0)')scln
                if(width.lt.0.0)scln = 1.0
            else if(name(1:5).eq.'-NOXL')then
            doxl = .false.
            else if(name(1:5).eq.'-NOYL')then
            doyl = .false.
            else if(name(1:5).eq.'-NOXN')then
            doxn = .false.
            else if(name(1:5).eq.'-NOYN')then
            doyn = .false.
            else if(name(1:6).eq.'-NDECX')then
            i = i + 1
            call mgtarg(i,name)
            read(name,'(i10)')ndecx
            else if(name(1:6).eq.'-NDECY')then
            i = i + 1
            call mgtarg(i,name)
            read(name,'(i10)')ndecy
            endif
            go to 11
   13 continue
c----
c       test for improper command line
c----
      return
      end
c
c
c
      function lstr(str)
c----
c     determine length of a string. Note f77 len gives declaration
c     we want length up to last non blank
c----
      integer*4 lstr
      character str*(*)
      l = len(str)
      do 100 i=l,1,-1
            lstr = i
            if(str(i:i) .ne. ' ')return
  100 continue
      return
      end

        subroutine lbox(x1,y1,string,ht,ipencolor)
        real*4 x1, y1, xoff, yoff, xw, yw,  ht
        character string*(*)
        integer*4 ipencolor
            ls = lstr(string)

            xl = x1 
            xu = x1 + ls * ht
            yl = y1 - 0.5*ht
            yu = y1 + ht
                
c-----
c           over write anything under the box
c-----
            call newpen(0)
            call shader(xl,yl,xu,yu,0,0,0.1,0.1)
            call newpen(ipencolor)
            if(ls.gt.0)then
                angle = 0.0
                call symbol(x1,y1,ht,string,angle,ls)
            endif
        return
        end

        subroutine rclip(xmin,xmax,ymin,ymax,xc1,yc1,xc2,yc2,
     1          x0,yy0,x1,yy1,iplt)
c-----
c       perform a line clipping for the plot region
c-----
c       xmin    R*4 - minimum value of X
c       xmax    R*4 - maximum value of X
c       ymin    R*4 - minimum value of Y
c       ymax    R*4 - maximum value of Y
c       xc1 R*4 - coordinate #1 of plotted line segment
c       yc1 R*4
c       xc2 R*4 - coordinate #2 of plotted line segment
c       yc2 R*4
c       x0  R*4 - first coordinate
c       yy0 R*4
c       x1  R*4 - second coordinate
c       yy1 R*4
c       iplt    I*4 - > 0 plot the segment, otherwise do not
c-----
        real*4 xmin, xmax, ymin,ymax
        real*4 xc1, yc1, xc2, yc2
        real*4 xx0, yz0, xx1, yz1
c-----
c       Fortran implementation of Cohen and Sutherland Line Clipping Routine
c-----
        integer c0, c1, c

        iplt = 0
        xx0 = x0
        yz0 = yy0
        xx1 = x1
        yz1 = yy1
        call linecode(xx0,yz0,ymax,xmin,ymin,xmax,c0)
        call linecode(xx1,yz1,ymax,xmin,ymin,xmax,c1)
        if( xx0 .ne. xx1)slope = (yz1-yz0)/(xx1-xx0)
        if( yz0 .ne. yz1) slopeinv = (xx1-xx0)/(yz1-yz0)
 1000   continue
        if(c0 .eq. 0 .and. c1 .eq.0)go to 1001
            if( (mod(c0,2).eq.1 .and. mod(c1,2).eq.1).or.
     1      (mod(c0/2,2).eq.1 .and. mod(c1/2,2).eq.1).or.
     2      (mod(c0/4,2).eq.1 .and. mod(c1/4,2).eq.1).or.
     3      (mod(c0/8,2).eq.1 .and. mod(c1/8,2).eq.1))return

                if(c0 .eq. c1) return 
                if(c0 .eq. 0 )then
                        c = c1
                else 
                        c = c0
            endif
                if(mod(c,2).eq.1)then
                        y = yz0 + slope*(xmin-xx0)
                        x = xmin
                endif
                if(mod(c/2,2).eq.1)then 
                y = yz0 + slope*(xmax-xx0)
                        x = xmax
                endif
                if(mod(c/8,2).eq.1)then
                        x = slopeinv*(ymax-yz0) + xx0
                        y = ymax
                endif
                if(mod(c/4,2).eq.1)then
                        x = slopeinv*(ymin-yz0) + xx0
                        y = ymin
                endif
                if(c .eq. c0)then
                        xx0 = x
                        yz0 = y
                call linecode(xx0,yz0,ymax,xmin,ymin,xmax,c0)
                else 
                        xx1 = x
                        yz1 = y
                call linecode(xx1,yz1,ymax,xmin,ymin,xmax,c1)
                endif
        go to 1000
 1001   continue
        xc1 = xx0
        xc2 = xx1
        yc1 = yz0
        yc2 = yz1
        iplt = 1
        return
        end
c       
        subroutine linecode(x,y,ymax,xmin,ymin,xmax,c)
        real*4 x, y, ymax, xmin, ymin, xmax
        integer*4 c
        c = 0

        if(x .lt. xmin) then
            c = 1
        else if(x .gt. xmax)then
            c = 2
        endif
        if(y .lt. ymin) then
            c = c + 4
        else if(y.gt.ymax)then
            c = c + 8
        endif
        return
        end

        function lgstr(str)
c-----
c       function to find the length of a string
c       this will only be used with file system path names
c       thus the first blank 
c       indicates the end of the string
c-----
        character*(*) str
        integer*4 lgstr
        n = len(str)
        lgstr = 1
        do 1000 i=n,1,-1
            lgstr = i
            if(str(i:i).ne.' ')goto 100
 1000   continue
  100   continue
        return
        end
